@@ -1,6 +1,6 @@
 /* i387-specific utility functions, for the remote server for GDB.
-   Copyright (C) 2000, 2001, 2002, 2005, 2007, 2008, 2009, 2010, 2011
-   Free Software Foundation, Inc.
+   Copyright (C) 2000-2002, 2005, 2007-2012 Free Software Foundation,
+   Inc.
 
    This file is part of GDB.
 
