@@ -1,7 +1,7 @@
 /* External/Public TUI Header File.
 
-   Copyright (C) 1998, 1999, 2000, 2001, 2004, 2007, 2008, 2009, 2010, 2011
-   Free Software Foundation, Inc.
+   Copyright (C) 1998-2001, 2004, 2007-2012 Free Software Foundation,
+   Inc.
 
    Contributed by Hewlett-Packard Company.
 
