@@ -13,8 +13,9 @@
  
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
+    Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA 02110-1301, USA. */
 
+#include "config.h"
 #include <stdio.h>
 #include <stdlib.h>
 
@@ -29,8 +30,18 @@
 typedef char *VoidStar;
 #endif
 
-typedef unsigned long ARMword;	/* must be 32 bits wide */
+#ifdef HAVE_STDINT_H
+#include <stdint.h>
+typedef uint32_t ARMword;
+typedef int32_t ARMsword;
+typedef uint64_t ARMdword;
+typedef int64_t ARMsdword;
+#else
+typedef unsigned int ARMword;	/* must be 32 bits wide */
+typedef signed int ARMsword;
 typedef unsigned long long ARMdword;	/* Must be at least 64 bits wide.  */
+typedef signed long long ARMsdword;
+#endif
 typedef struct ARMul_State ARMul_State;
 
 typedef unsigned ARMul_CPInits (ARMul_State * state);
@@ -134,6 +145,7 @@ struct ARMul_State
   unsigned is_v4;		/* Are we emulating a v4 architecture (or higher) ?  */
   unsigned is_v5;		/* Are we emulating a v5 architecture ?  */
   unsigned is_v5e;		/* Are we emulating a v5e architecture ?  */
+  unsigned is_v6;		/* Are we emulating a v6 architecture ?  */
   unsigned is_XScale;		/* Are we emulating an XScale architecture ?  */
   unsigned is_iWMMXt;		/* Are we emulating an iWMMXt co-processor ?  */
   unsigned is_ep9312;		/* Are we emulating a Cirrus Maverick co-processor ?  */
@@ -166,6 +178,7 @@ struct ARMul_State
 #define ARM_XScale_Prop  0x200
 #define ARM_ep9312_Prop  0x400
 #define ARM_iWMMXt_Prop  0x800
+#define ARM_v6_Prop      0x1000
 
 /***************************************************************************\
 *                   Macros to extract instruction fields                    *
