@@ -1,6 +1,6 @@
 /* frv simulator support code
-   Copyright (C) 1998, 2000, 2001, 2007, 2008, 2009, 2010, 2011
-   Free Software Foundation, Inc.
+   Copyright (C) 1998, 2000-2001, 2007-2012 Free Software Foundation,
+   Inc.
    Contributed by Red Hat.
 
 This file is part of the GNU simulators.
