@@ -45,6 +45,7 @@
 
 #include <ctype.h>
 
+#include "libiberty.h"
 
 /* manipulate/lookup device names */
 
@@ -539,14 +540,14 @@ parse_reg_property(device *current,
     if (device_nr_size_cells(bus) > 0)
       chp = parse_size(current, bus, chp, &regs[reg_nr].size);
     else
-      memset(&regs[reg_nr].size, sizeof (&regs[reg_nr].size), 0);
+      memset(&regs[reg_nr].size, 0, sizeof (&regs[reg_nr].size));
   }
 
   /* create it */
   device_add_reg_array_property(current, property_name,
 				regs, nr_regs);
 
-  zfree(regs);
+  free(regs);
 }
 
 
@@ -583,7 +584,7 @@ parse_ranges_property(device *current,
   /* create it */
   device_add_range_array_property(current, property_name, ranges, nr_ranges);
 
-  zfree(ranges);
+  free(ranges);
 }
 
 
@@ -625,6 +626,50 @@ parse_integer_property(device *current,
   }
 }
 
+/* PROPERTY_VALUE is a raw property value.  Quote it as required by
+   parse_string_property.  It is the caller's responsibility to free
+   the memory returned.  */
+
+EXTERN_TREE\
+(char *)
+tree_quote_property(const char *property_value)
+{
+  char *p;
+  char *ret;
+  const char *chp;
+  int quotees;
+
+  /* Count characters needing quotes in PROPERTY_VALUE.  */
+  quotees = 0;
+  for (chp = property_value; *chp; ++chp)
+    if (*chp == '\\' || *chp == '"')
+      ++quotees;
+  
+  ret = (char *) xmalloc (strlen (property_value) 
+			  + 2 /* quotes */
+			  + quotees
+			  + 1 /* terminator */);
+
+  p = ret;
+  /* Add the opening quote.  */
+  *p++ = '"';
+  /* Copy the value.  */
+  for (chp = property_value; *chp; ++chp)
+    if (*chp == '\\' || *chp == '"')
+      {
+	/* Quote this character.  */ 
+	*p++ = '\\';
+	*p++ = *chp;
+      }
+    else
+      *p++ = *chp;
+  /* Add the closing quote.  */
+  *p++ = '"';
+  /* Terminate the string.  */
+  *p++ = '\0';
+
+  return ret;
+}
 
 /* <string> ... */
 
@@ -725,9 +770,9 @@ parse_string_property(device *current,
   /* flush the created string */
   while (nr_strings > 0) {
     nr_strings--;
-    zfree(strings[nr_strings]);
+    free(strings[nr_strings]);
   }
-  zfree(strings);
+  free(strings);
 }
 
 
